# SPDX-FileCopyrightText: Magenta ApS <https://magenta.dk>
# SPDX-License-Identifier: MPL-2.0
from collections.abc import Mapping
from inspect import signature
from typing import Any
from typing import Callable
from typing import Iterator
from typing import Optional


class LazyEval:
    """Lazily evaluated dict member, used in tandem with `LazyDict`.

    For details and usage see `LazyDict`.
    """

    def __init__(self, cally: Callable, cache: bool = True) -> None:
        """Initializer.

        *Note: If the callable needs arguments, these should be provided before
               calling this initializer using `functools.partial` or similar.
               Alternatively arguments can be provided and fetched from the `LazyDict`
               itself, as this is provided during the `__call__` method.*

        Args:
            cally: The callable to execute for lazy evaluation.
            cache: Whether to cache the result in the `LazyDict` after execution.

        Returns:
            The `LazyEval` to be loaded into a `LazyDict`.
        """

        self.cally = cally
        self.cache = cache

    def do_cache(self) -> bool:
        """Check the caching strategy used.

        Returns:
            Whether the return-value of the `__call__` method should be cached.
        """
        return self.cache

    def __call__(self, key: Any, dictionary: "LazyDict") -> Any:
        """Evaluate the callable.

        Args:
            key: The key to which we are assigning this value.
            dictionary: The dictionary to which we are assigning this value.

        Returns:
            Whatever is returned by the callable `cally`.
        """
        return self.cally(key, dictionary)


def LazyEvalDerived(cally: Callable, cache: bool = True) -> LazyEval:
    """Create a 'derived' `LazyEval`.

    Derived means that the cally callable will receive arguments from the dictionary
    derived by the parameters it takes.

    Example:
        ```Python
        lazy_dict = LazyDict({"base_value": 5})
        lazy_dict["derived_val"] = LazyEvalDerived(
            lambda base_value: base_value
        )
        assert lazy_dict["derived_val"] == 5
        ```

    Args:
        cally: Callable taking any number of arguments.
               Argument names must match other `LazyDict` entries.
        cache: Same meaning as for `LazyEval`.

    Returns:
        A `LazyEval` set to evaluate a wrapper function that reads out values
        from the `LazyDict` according to the parameters taken by `cally`, before
        calling `cally` with these values.
    """
    cally_signature = signature(cally)

    def inner(key: Any, dictionary: "LazyDict") -> Any:
        return cally(
            **{varname: dictionary[varname] for varname in cally_signature.parameters}
        )

    return LazyEval(inner, cache)


def LazyEvalBare(cally: Callable, cache: bool = True) -> LazyEval:
    """Create a 'bare' `LazyEval`.

    Bare means that the cally callable will not receive any arguments.

    Example:
        ```Python
        lazy_dict = LazyDict({"base_value": 5})
        lazy_dict["bare_val"] = LazyEvalBare(
            lambda: 3.14
        )
        assert lazy_dict["bare_val"] == 3.14
        ```

    Args:
        cally: Callable taking no arguments.
        cache: Same meaning as for `LazyEval`.

    Returns:
        A `LazyEval` set to evaluate a wrapper function that calls `cally`
        without arguments.
    """

    def inner(key: Any, dictionary: "LazyDict") -> Any:
        """Throw away the arguments, and call the callable."""
        return cally()

    return LazyEval(inner, cache)


class LazyDict(Mapping):
    """Dictionary supporting lazy evaluation of some keys.

    Example:
        ```Python
        def expensive_func(n: int = 5):
            time.sleep(n)
            return n

        # Initialization finishes without sleeping
        d = LazyDicy({'a': LazyEval(expensive_func), 'b': 2})
        print(d['b'])  # --> Prints 2 immediately
        print(d['a'])  # --> Prints 5 after 5 seconds
        print(d['a'])  # --> Prints 5 immediately (cached)
        ```
    """

    def __init__(self, *args: Any, **kwargs: Optional[Any]) -> None:
        """Initialize the internal dictionary with `*args` and `**kwargs` directly.

        Args:
            args: Arguments for internal dictionary.
            kwargs: Keyword arguments for internal dictionary.

        Returns:
            dictionary ready to be used as an ordinary `dict`.
        """
        self._raw_dict = dict(*args, **kwargs)

    def __getitem__(self, key: Any) -> Any:
        """Implementation of evaluation of self[key].

        Fetches a value from the underlying dictionary, if the value is a `LazyEval`
        it is evaluated using `_handle_lazy` method, otherwise it is returned as-is.

        Args:
            key: The key to lookup

        Returns:
            If the value stored at the `key` location is, a `LazyEval`:

            * it is evaluated and its value is returned (potentially caching it),
            * otherwise the value from the internal dictionary is returned.
        """
        value = self._raw_dict.__getitem__(key)
        # Check if we got back a LazyEval item
        if isinstance(value, LazyEval):
            return self._handle_lazy(key, value)
        return value

    def __setitem__(self, key: Any, value: Any) -> None:
        self._raw_dict.__setitem__(key, value)

    def _handle_lazy(self, key: Any, lazy_eval: LazyEval) -> Any:
        """Evaluate the `LazyEval` and cache result if configured to do so."""
        value = lazy_eval(key, self)
        if lazy_eval.do_cache():
            self._raw_dict[key] = value
        return value

    def __str__(self) -> str:
        return str(self._raw_dict)

    def __repr__(self) -> str:
        return repr(self._raw_dict)

    def __iter__(self) -> Iterator:
        return iter(self._raw_dict)

    def __len__(self) -> int:
        return len(self._raw_dict)



# SPDX-FileCopyrightText: Magenta ApS <https://magenta.dk>
# SPDX-License-Identifier: MPL-2.0
import time
from functools import lru_cache
from typing import Any
from typing import Dict
from typing import Optional
from typing import Tuple
from warnings import warn

import requests
from pydantic import AnyHttpUrl
from pydantic import BaseSettings
from pydantic import Field
from pydantic import parse_obj_as
from pydantic import root_validator


# Exception
class AuthError(Exception):
    """Raised when errors in authentication occurs."""


# Settings


class TokenSettings(BaseSettings):
    """Collection of settings required for authentication against OS2mo.

    Example:
        ```Python
        import requests
        from fastramqpi.ra_utils.headers import TokenSettings

        session = requests.Session()
        session.headers = TokenSettings().get_headers()
        response = session.get("https://moratest.magenta.dk/service/o/")
        response.raise_for_status()
        print(response.json())
        ```
    """

    client_id: str = "mo"
    client_secret: Optional[str]  # in the future, this should be required
    auth_realm: str = "mo"
    auth_server: AnyHttpUrl = Field(
        parse_obj_as(AnyHttpUrl, "http://localhost:8081/auth")
    )
    saml_token: Optional[str]  # deprecate when fully on keycloak?

    # Re-new token this many seconds before it actually expires
    oidc_token_lifespan_offset: int = 30

    class Config:
        frozen = True

    @root_validator
    def validate_settings(cls, values: Dict[str, Any]) -> Dict[str, Any]:
        """Validate token settings by checking that either `client_secret`,
        `saml_token`, or both exist.

        This validation occurs when `TokenSettings` are initialized and does
        not mutate values.

        Args:
            values: Initialized `TokenSettings` values.

        Raises:
            UserWarning: If none of `CLIENT_SECRET` or `SAML_TOKEN` are given
                during initialisation.
            PendingDeprecationWarning: If `SAML_TOKEN` is used.

        Returns:
            `TokenSettings` values, unmodified.
        """

        keycloak, saml = values.get("client_secret"), values.get("saml_token")
        if not any([keycloak, saml]):
            warn("No secret or token given", stacklevel=2)
        if saml:
            warn(
                "Using SAML tokens will be deprecated",
                PendingDeprecationWarning,
                stacklevel=2,
            )
        return values

    @lru_cache(maxsize=None)
    def _fetch_keycloak_token(self) -> Tuple[float, str]:
        """Fetch a keycloak token and its expiry time.

        Raises:
            AuthError: If no client secret is given or the response from
                the authentication server raises an error.

        Returns:
            Tuple of token-expiry time in seconds and the token itself.
        """
        token_url = (
            f"{self.auth_server}/realms/{self.auth_realm}/protocol/openid-connect/token"
        )
        if self.client_secret is None:
            raise AuthError("No client secret given")
        payload: Dict[str, str] = {
            "grant_type": "client_credentials",
            "client_id": self.client_id,
            "client_secret": self.client_secret,
        }
        try:
            response = requests.post(token_url, data=payload)
            response.raise_for_status()
        except requests.RequestException as err:
            raise AuthError(f"Failed to get Keycloak token: {err}")
        response_payload: Dict[str, Any] = response.json()
        expires: int = response_payload["expires_in"]
        token: str = response_payload["access_token"]
        return time.monotonic() + float(expires), token

    def _fetch_bearer(self, force: bool = False, logger: Any = None) -> str:
        """Fetch a Keycloak bearer token.

        Automatically refetches the token after it expires.

        Args:
            force: always refresh token if true
            logger: logger used for logging token refresh info

        Raises:
            AuthError: If no client secret is given or the response from
                the authentication server raises an error.

        Returns:
            The Bearer token itself.
        """
        expires, token = self._fetch_keycloak_token()
        if force or expires - self.oidc_token_lifespan_offset < time.monotonic():
            self._fetch_keycloak_token.cache_clear()
            expires, token = self._fetch_keycloak_token()
            if logger:
                logger.debug("New token fetched", expires=expires, token=token)
        return "Bearer " + token

    def get_headers(self, force: bool = False, logger: Any = None) -> Dict[str, str]:
        """Get authorization headers based on configured tokens.

        If both a client secret and a SAML token are configured,
        they will both exist in the headers dict, with their respective keys:

        * `Authorization: Bearer ${TOKEN}`, and
        * `Session: ${TOKEN}`

        Args:
            force: always refresh token if true
            logger: logger used for logging token refresh info

        Raises:
            AuthError: If `client_secret` is given, but the response from the
                authentication server raises an error.

        Returns:
            Header dictionary.
        """
        headers: Dict[str, str] = {}
        if self.saml_token:
            headers["Session"] = self.saml_token
        if self.client_secret:
            headers["Authorization"] = self._fetch_bearer(force, logger)
        return headers



# SPDX-FileCopyrightText: Magenta ApS <https://magenta.dk>
# SPDX-License-Identifier: MPL-2.0
import re
from functools import lru_cache
from typing import Any
from typing import Callable
from typing import Dict
from typing import Iterator
from typing import Pattern

from pydantic import BaseModel

# Regex from https://semver.org/
_semver_regex = (
    # Version part
    r"^(?P<major>0|[1-9]\d*)\.(?P<minor>0|[1-9]\d*)\.(?P<patch>0|[1-9]\d*)"
    # Prerelease part
    r"(?:-(?P<prerelease>(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)"
    r"(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?"
    # Build metadata
    r"(?:\+(?P<buildmetadata>[0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$"
)


@lru_cache
def get_regex() -> Pattern:
    return re.compile(_semver_regex)


class SemanticVersion(str):
    """Pydantic field model for validating semantic versioning.

    See: <a href="https://semver.org/">semver.org</a> for details.

    Example:
        ```Python
        version = "1.1.2+meta-valid"
        SemanticVersion.validate(version)
        ```
    """

    @classmethod
    def __get_validators__(cls) -> Iterator[Callable]:
        yield cls.validate

    @classmethod
    def __modify_schema__(cls, field_schema: Dict) -> None:
        field_schema.update(
            pattern=_semver_regex,
            examples=["0.1.0", "1.0.0-alpha", "1.0.0-alpha+001"],
        )

    @classmethod
    def validate(cls, v: Any) -> "SemanticVersion":
        """Validate that `v` is a valid semantic version.

        Args:
            v: Value to validate against semver regex.

        Raises:
            TypeError: If `v` is not a string
            ValueError: If `v` does not match the semver regex.

        Returns:
            Validated field model.
        """
        if not isinstance(v, str):
            raise TypeError("string required")
        m = get_regex().fullmatch(v)
        if not m:
            raise ValueError("invalid semver format")
        return cls(v)

    def __repr__(self) -> str:
        return f"SemanticVersion({super().__repr__()})"


class SemanticVersionModel(BaseModel):
    """Pydantic model for validating semantic versioning.

    See: <a href="https://semver.org/">semver.org</a> for details.

    Example:
        ```Python
        version = "1.1.2+meta-valid"
        SemanticVersionModel(__root__=version)
        ```

    Example:
        ```Python
        schema = SemanticVersionModel.schema()
        print(json.dumps(schema))
        ```
        Which yields:
        ```Json
        {
            "title": "SemanticVersionModel",
            "description": "...",
            "pattern": "^(?P<major>0|[1-9]\\d*)...$",
            "examples": [
                "0.1.0",
                "1.0.0-alpha",
                "1.0.0-alpha+001"
            ],
            "type": "string"
        }
        ```
    """

    __root__: SemanticVersion



from ._testing__create_employee import TestingCreateEmployee
from ._testing__create_employee import TestingCreateEmployeeEmployeeCreate
from ._testing__create_employee import TestingCreateEmployeeEmployeeCreateValidities
from ._testing__get_employee import TestingGetEmployee
from ._testing__get_employee import TestingGetEmployeeEmployees
from ._testing__get_employee import TestingGetEmployeeEmployeesObjects
from ._testing__get_employee import TestingGetEmployeeEmployeesObjectsValidities
from ._testing__send_event import TestingSendEvent
from .acknowledge_event import AcknowledgeEvent
from .async_base_client import AsyncBaseClient
from .base_model import BaseModel
from .client import GraphQLClient
from .declare_event_listener import DeclareEventListener
from .declare_event_listener import DeclareEventListenerEventListenerDeclare
from .declare_event_namespace import DeclareEventNamespace
from .declare_event_namespace import DeclareEventNamespaceEventNamespaceDeclare
from .enums import AccessLogModel
from .enums import FileStore
from .enums import OwnerInferencePriority
from .exceptions import GraphQLClientError
from .exceptions import GraphQLClientGraphQLError
from .exceptions import GraphQLClientGraphQLMultiError
from .exceptions import GraphQLClientHttpError
from .exceptions import GraphQlClientInvalidResponseError
from .fetch_event import FetchEvent
from .fetch_event import FetchEventEventFetch
from .input_types import AccessLogFilter
from .input_types import AddressCreateInput
from .input_types import AddressFilter
from .input_types import AddressRegistrationFilter
from .input_types import AddressTerminateInput
from .input_types import AddressUpdateInput
from .input_types import AssociationCreateInput
from .input_types import AssociationFilter
from .input_types import AssociationRegistrationFilter
from .input_types import AssociationTerminateInput
from .input_types import AssociationUpdateInput
from .input_types import ClassCreateInput
from .input_types import ClassFilter
from .input_types import ClassOwnerFilter
from .input_types import ClassRegistrationFilter
from .input_types import ClassTerminateInput
from .input_types import ClassUpdateInput
from .input_types import ConfigurationFilter
from .input_types import DescendantParentBoundOrganisationUnitFilter
from .input_types import EmployeeCreateInput
from .input_types import EmployeeFilter
from .input_types import EmployeeRegistrationFilter
from .input_types import EmployeesBoundAddressFilter
from .input_types import EmployeesBoundAssociationFilter
from .input_types import EmployeesBoundEngagementFilter
from .input_types import EmployeesBoundITUserFilter
from .input_types import EmployeesBoundLeaveFilter
from .input_types import EmployeesBoundManagerFilter
from .input_types import EmployeeTerminateInput
from .input_types import EmployeeUpdateInput
from .input_types import EngagementCreateInput
from .input_types import EngagementFilter
from .input_types import EngagementRegistrationFilter
from .input_types import EngagementTerminateInput
from .input_types import EngagementUpdateInput
from .input_types import EventAcknowledgeInput
from .input_types import EventFilter
from .input_types import EventSendInput
from .input_types import EventSilenceInput
from .input_types import EventUnsilenceInput
from .input_types import FacetCreateInput
from .input_types import FacetFilter
from .input_types import FacetRegistrationFilter
from .input_types import FacetsBoundClassFilter
from .input_types import FacetTerminateInput
from .input_types import FacetUpdateInput
from .input_types import FileFilter
from .input_types import FullEventFilter
from .input_types import HealthFilter
from .input_types import ITAssociationCreateInput
from .input_types import ITAssociationTerminateInput
from .input_types import ITAssociationUpdateInput
from .input_types import ITSystemCreateInput
from .input_types import ITSystemFilter
from .input_types import ITSystemRegistrationFilter
from .input_types import ITSystemTerminateInput
from .input_types import ITSystemUpdateInput
from .input_types import ItuserBoundAddressFilter
from .input_types import ItuserBoundRoleBindingFilter
from .input_types import ITUserCreateInput
from .input_types import ITUserFilter
from .input_types import ITUserRegistrationFilter
from .input_types import ITUserTerminateInput
from .input_types import ITUserUpdateInput
from .input_types import KLECreateInput
from .input_types import KLEFilter
from .input_types import KLERegistrationFilter
from .input_types import KLETerminateInput
from .input_types import KLEUpdateInput
from .input_types import LeaveCreateInput
from .input_types import LeaveFilter
from .input_types import LeaveRegistrationFilter
from .input_types import LeaveTerminateInput
from .input_types import LeaveUpdateInput
from .input_types import ListenerCreateInput
from .input_types import ListenerDeleteInput
from .input_types import ListenerFilter
from .input_types import ListenersBoundFullEventFilter
from .input_types import ManagerCreateInput
from .input_types import ManagerFilter
from .input_types import ManagerRegistrationFilter
from .input_types import ManagerTerminateInput
from .input_types import ManagerUpdateInput
from .input_types import ModelsUuidsBoundRegistrationFilter
from .input_types import NamespaceCreateInput
from .input_types import NamespaceDeleteInput
from .input_types import NamespaceFilter
from .input_types import NamespacesBoundListenerFilter
from .input_types import OrganisationCreate
from .input_types import OrganisationUnitCreateInput
from .input_types import OrganisationUnitFilter
from .input_types import OrganisationUnitRegistrationFilter
from .input_types import OrganisationUnitTerminateInput
from .input_types import OrganisationUnitUpdateInput
from .input_types import OrgUnitsboundaddressfilter
from .input_types import OrgUnitsboundassociationfilter
from .input_types import OrgUnitsboundengagementfilter
from .input_types import OrgUnitsboundituserfilter
from .input_types import OrgUnitsboundklefilter
from .input_types import OrgUnitsboundleavefilter
from .input_types import OrgUnitsboundmanagerfilter
from .input_types import OrgUnitsboundrelatedunitfilter
from .input_types import OwnerCreateInput
from .input_types import OwnerFilter
from .input_types import OwnerTerminateInput
from .input_types import OwnerUpdateInput
from .input_types import ParentsBoundClassFilter
from .input_types import ParentsBoundFacetFilter
from .input_types import ParentsBoundOrganisationUnitFilter
from .input_types import RAOpenValidityInput
from .input_types import RAValidityInput
from .input_types import RegistrationFilter
from .input_types import RelatedUnitFilter
from .input_types import RelatedUnitsUpdateInput
from .input_types import RoleBindingCreateInput
from .input_types import RoleBindingFilter
from .input_types import RoleBindingTerminateInput
from .input_types import RoleBindingUpdateInput
from .input_types import RoleRegistrationFilter
from .input_types import UuidsBoundClassFilter
from .input_types import UuidsBoundEmployeeFilter
from .input_types import UuidsBoundEngagementFilter
from .input_types import UuidsBoundFacetFilter
from .input_types import UuidsBoundITSystemFilter
from .input_types import UuidsBoundITUserFilter
from .input_types import UuidsBoundLeaveFilter
from .input_types import UuidsBoundOrganisationUnitFilter
from .input_types import ValidityInput

__all__ = [
    "AccessLogFilter",
    "AccessLogModel",
    "AcknowledgeEvent",
    "AddressCreateInput",
    "AddressFilter",
    "AddressRegistrationFilter",
    "AddressTerminateInput",
    "AddressUpdateInput",
    "AssociationCreateInput",
    "AssociationFilter",
    "AssociationRegistrationFilter",
    "AssociationTerminateInput",
    "AssociationUpdateInput",
    "AsyncBaseClient",
    "BaseModel",
    "ClassCreateInput",
    "ClassFilter",
    "ClassOwnerFilter",
    "ClassRegistrationFilter",
    "ClassTerminateInput",
    "ClassUpdateInput",
    "ConfigurationFilter",
    "DeclareEventListener",
    "DeclareEventListenerEventListenerDeclare",
    "DeclareEventNamespace",
    "DeclareEventNamespaceEventNamespaceDeclare",
    "DescendantParentBoundOrganisationUnitFilter",
    "EmployeeCreateInput",
    "EmployeeFilter",
    "EmployeeRegistrationFilter",
    "EmployeeTerminateInput",
    "EmployeeUpdateInput",
    "EmployeesBoundAddressFilter",
    "EmployeesBoundAssociationFilter",
    "EmployeesBoundEngagementFilter",
    "EmployeesBoundITUserFilter",
    "EmployeesBoundLeaveFilter",
    "EmployeesBoundManagerFilter",
    "EngagementCreateInput",
    "EngagementFilter",
    "EngagementRegistrationFilter",
    "EngagementTerminateInput",
    "EngagementUpdateInput",
    "EventAcknowledgeInput",
    "EventFilter",
    "EventSendInput",
    "EventSilenceInput",
    "EventUnsilenceInput",
    "FacetCreateInput",
    "FacetFilter",
    "FacetRegistrationFilter",
    "FacetTerminateInput",
    "FacetUpdateInput",
    "FacetsBoundClassFilter",
    "FetchEvent",
    "FetchEventEventFetch",
    "FileFilter",
    "FileStore",
    "FullEventFilter",
    "GraphQLClient",
    "GraphQLClientError",
    "GraphQLClientGraphQLError",
    "GraphQLClientGraphQLMultiError",
    "GraphQLClientHttpError",
    "GraphQlClientInvalidResponseError",
    "HealthFilter",
    "ITAssociationCreateInput",
    "ITAssociationTerminateInput",
    "ITAssociationUpdateInput",
    "ITSystemCreateInput",
    "ITSystemFilter",
    "ITSystemRegistrationFilter",
    "ITSystemTerminateInput",
    "ITSystemUpdateInput",
    "ITUserCreateInput",
    "ITUserFilter",
    "ITUserRegistrationFilter",
    "ITUserTerminateInput",
    "ITUserUpdateInput",
    "ItuserBoundAddressFilter",
    "ItuserBoundRoleBindingFilter",
    "KLECreateInput",
    "KLEFilter",
    "KLERegistrationFilter",
    "KLETerminateInput",
    "KLEUpdateInput",
    "LeaveCreateInput",
    "LeaveFilter",
    "LeaveRegistrationFilter",
    "LeaveTerminateInput",
    "LeaveUpdateInput",
    "ListenerCreateInput",
    "ListenerDeleteInput",
    "ListenerFilter",
    "ListenersBoundFullEventFilter",
    "ManagerCreateInput",
    "ManagerFilter",
    "ManagerRegistrationFilter",
    "ManagerTerminateInput",
    "ManagerUpdateInput",
    "ModelsUuidsBoundRegistrationFilter",
    "NamespaceCreateInput",
    "NamespaceDeleteInput",
    "NamespaceFilter",
    "NamespacesBoundListenerFilter",
    "OrgUnitsboundaddressfilter",
    "OrgUnitsboundassociationfilter",
    "OrgUnitsboundengagementfilter",
    "OrgUnitsboundituserfilter",
    "OrgUnitsboundklefilter",
    "OrgUnitsboundleavefilter",
    "OrgUnitsboundmanagerfilter",
    "OrgUnitsboundrelatedunitfilter",
    "OrganisationCreate",
    "OrganisationUnitCreateInput",
    "OrganisationUnitFilter",
    "OrganisationUnitRegistrationFilter",
    "OrganisationUnitTerminateInput",
    "OrganisationUnitUpdateInput",
    "OwnerCreateInput",
    "OwnerFilter",
    "OwnerInferencePriority",
    "OwnerTerminateInput",
    "OwnerUpdateInput",
    "ParentsBoundClassFilter",
    "ParentsBoundFacetFilter",
    "ParentsBoundOrganisationUnitFilter",
    "RAOpenValidityInput",
    "RAValidityInput",
    "RegistrationFilter",
    "RelatedUnitFilter",
    "RelatedUnitsUpdateInput",
    "RoleBindingCreateInput",
    "RoleBindingFilter",
    "RoleBindingTerminateInput",
    "RoleBindingUpdateInput",
    "RoleRegistrationFilter",
    "TestingCreateEmployee",
    "TestingCreateEmployeeEmployeeCreate",
    "TestingCreateEmployeeEmployeeCreateValidities",
    "TestingGetEmployee",
    "TestingGetEmployeeEmployees",
    "TestingGetEmployeeEmployeesObjects",
    "TestingGetEmployeeEmployeesObjectsValidities",
    "TestingSendEvent",
    "UuidsBoundClassFilter",
    "UuidsBoundEmployeeFilter",
    "UuidsBoundEngagementFilter",
    "UuidsBoundFacetFilter",
    "UuidsBoundITSystemFilter",
    "UuidsBoundITUserFilter",
    "UuidsBoundLeaveFilter",
    "UuidsBoundOrganisationUnitFilter",
    "ValidityInput",
]
